import * as GS from 'gensequence';
function toMatchRangeWithText(m) {
    const index = m.index || 0;
    const _text = m[0];
    return {
        startPos: index,
        endPos: index + _text.length,
        text: _text,
    };
}
export function findMatchingRanges(pattern, text) {
    if (pattern.source === '.*') {
        return [{ startPos: 0, endPos: text.length }];
    }
    const regex = new RegExp(pattern);
    if (!regex.global) {
        const m = text.match(regex);
        if (!m)
            return [];
        return [toMatchRangeWithText(m)];
    }
    return [...text.matchAll(regex)].map(toMatchRangeWithText);
}
function compareRanges(a, b) {
    return a.startPos - b.startPos || a.endPos - b.endPos;
}
export function unionRanges(ranges) {
    return makeSortedMatchRangeArray([..._unionRanges(ranges)]);
}
function* _unionRanges(ranges) {
    const sortedRanges = sortMatchRangeArray(ranges);
    if (!sortedRanges.length)
        return;
    let { startPos, endPos } = sortedRanges[0];
    for (const r of ranges) {
        if (r.startPos > endPos) {
            yield { startPos, endPos };
            startPos = r.startPos;
            endPos = r.endPos;
            continue;
        }
        endPos = Math.max(endPos, r.endPos);
    }
    if (startPos < endPos) {
        yield { startPos, endPos };
    }
}
export function findMatchingRangesForPatterns(patterns, text) {
    const matchedPatterns = GS.genSequence(patterns).concatMap((pattern) => findMatchingRanges(pattern, text));
    return unionRanges(matchedPatterns.toArray());
}
/**
 * Create a new set of positions that have the excluded position ranges removed.
 */
export function excludeRanges(includeRanges, excludeRanges) {
    return [..._excludeRanges(sortMatchRangeArray(includeRanges), sortMatchRangeArray(excludeRanges))];
}
function* _excludeRanges(includeRanges, excludeRanges) {
    if (!includeRanges.length)
        return;
    if (!excludeRanges.length) {
        yield* includeRanges;
        return;
    }
    let exIndex = 0;
    const limit = excludeRanges.length;
    for (const incRange of includeRanges) {
        const endPos = incRange.endPos;
        let startPos = incRange.startPos;
        for (; exIndex < limit; ++exIndex) {
            const ex = excludeRanges[exIndex];
            if (ex.startPos >= endPos)
                break;
            if (ex.endPos <= startPos)
                continue;
            if (ex.startPos > startPos) {
                yield { startPos, endPos: ex.startPos };
            }
            startPos = ex.endPos;
            if (startPos >= endPos)
                break;
        }
        if (startPos < endPos) {
            yield { startPos, endPos };
        }
    }
}
export function extractRangeText(text, ranges) {
    return ranges.map(({ startPos, endPos }) => ({
        startPos,
        endPos,
        text: text.slice(startPos, endPos),
    }));
}
const SymSortedMatchRangeArray = Symbol('SortedMatchRangeArray');
function sortMatchRangeArray(values) {
    if (isSortedMatchRangeArray(values))
        return values;
    return makeSortedMatchRangeArray(values.sort(compareRanges));
}
function isSortedMatchRangeArray(a) {
    return a[SymSortedMatchRangeArray] === true;
}
function makeSortedMatchRangeArray(sortedValues) {
    const sorted = sortedValues;
    sorted[SymSortedMatchRangeArray] = true;
    Object.freeze(sorted);
    return sorted;
}
export const __testing__ = {
    makeSortedMatchRangeArray,
};
//# sourceMappingURL=TextRange.js.map